#include <math.h>
#include <stdio.h>
#include "natunits.h"
#include "qcdeqn.h"

#define lms 364.834 /* schaalfactor labda_MS 379 voor eerste orde*/
#define Nf 3.0 /*Model voor Nf=3 smaken */
#define Nc 3.0 /*Model voor Nc=3 kleuren */

double zero=1.0;
double B0=(11.0 - 2.0*Nf/3.0),B1=(51.0 - 19.0*Nf/3.0);
double B2=(2857.0 - 5033.0*Nf/9.0 + 325.0*Nf*Nf/27.0);

/* Functie voor running coupling */
double alpha (double labda)
{
	double u,first,second;
	u = 2.0*log(labda/lms);
	first = -2.0*((B1/(B0*B0))*(log(u)/u));
	second = (4.0*B1*B1)/(pow(B0,4.0)*u*u) * (pow((log(u)-(1.0/2.0)),2.0) + (B2*B0)/(8.0*B1*B1)-(5.0/4.0));
	return((4.0*PI)/(B0*u)*(1.0+first+second));
}

double dalpha (double labda)
{
	double dadu,dudl,u;
	u = 2.0*log(labda/lms);
	dadu = - 1.0/(pow(B0,5.0)*pow(u,4.0))*(2.0*PI*(2.0*u*u*pow(B0,4.0) - 4.0*u*(2.0*log(u) - 1.0)*B1*B0*B0 + 3.0*B2*B0 + 8.0*(log(u)-2.0) * (3.0 * log(u) + 1.0)*B1*B1));
	dudl = 2.0/labda;	
	return (dadu*dudl*lschaal); 
}

/* Thermodynamische potentiaal (massaloos, 1e orde) */
double thpot(double mu)
{
        double antw;
	antw = -(Nf*pow(mu,4.0))/(4.0*PI*PI)*(zero+first(mu)+second(mu));
	return antw;
}

double number(double mu)
{
        double part1,part2,part3,prefac,result;
	double Gg = 10.374 - 0.536 * (Nf) + (Nf) * log(Nf);
	/* Eerste deel is afgeleide naar mu, 2e deel is afgeleide naar alpha via kettingregel: dfunc/dalpha*dalpha/dlabda*dlabda/dmu. dlabda/dbu=lschaal  */
	part1 = (Nf*pow(mu,3.0)/(PI*PI))*(zero+first(mu)+second(mu));
	if(inter){
		prefac = (Nf*pow(mu,4.0))/(4.0*PI*PI);
		part2 = -2.0*dalpha(lschaal*mu)/PI;
		part3 = -(1.0/(3.0*PI*PI)) * (((66.0-4.0*Nf) * log(lschaal) + 3.0 * (2.0 * Gg + Nf + 2.0 * Nf * log(alpha(lschaal * mu)/PI))) * alpha(lschaal * mu)) * dalpha(lschaal * mu);
	}
	else{
	prefac=0.0;
	part2=0.0;
	part3=0.0;	
	}
	result = part1+prefac*(part2+part3);
	return result;
}

double epsf(double mu)
{
	double ans,p,nr;
	p = -thpot(mu);
	nr = number(mu);
	ans = -p + mu * nr;
	return ans;
}

/* Toestandsvergelijking */
double mufunc(double mu, double p)
{
	double ans;
	ans=-thpot(mu)-p;
	return ans;
	
}

double first (double mu)
{
	double antw=0.0;
	if(inter){antw =  -2.0*(alpha(lschaal*mu)/PI);}
	else{antw = 0.0;}
	return antw;
}

double second (double mu)
{
	double antw=0.0;
	double Gg = 10.374 - 0.536 * (Nf) + (Nf) * log(Nf);
	if(inter) {
		antw = -(Gg+Nf*log(alpha(lschaal*mu)/PI) + (11.0-(2.0/3.0)*Nf)*log(lschaal))*pow((alpha(lschaal*mu)/PI),2.0);
	}
	return antw;
}
